@@ -203,7 +203,7 @@ class TrainingPhase(utils.repr._ReprMixin):
 
     Args:
         name: The name of the training phase.
-        n_epochs: The number of epochs to run in this phase. In standard
+        epochs: The number of epochs to run in this phase. In standard
             item-based sampling, the model sees every item once per epoch
             In the case of negative edge sampling, this is not guaranteed, and
             an epoch instead comprises ``batches_per_epoch`` batches (see
@@ -242,7 +242,7 @@ class TrainingPhase(utils.repr._ReprMixin):
 
     def __init__(self,
         name: Optional[str] = None,
-        n_epochs: int = 5,
+        epochs: int = 5,
         batch_size: int = 50,
         batches_per_epoch: int = -1,
         sampling: Literal['standard', 'negative_edge'] = 'standard',
@@ -256,7 +256,7 @@ class TrainingPhase(utils.repr._ReprMixin):
     ):
         
         self.name = name
-        self.n_epochs = n_epochs
+        self.epochs = epochs
         self.batch_size = batch_size
         self.batches_per_epoch = batches_per_epoch
         self.sampling = sampling
@@ -274,9 +274,7 @@ class TrainingPhase(utils.repr._ReprMixin):
         self.report_interval = report_interval
         self.kwargs = kwargs
 
-        try:
-            self.optimizer([torch.tensor([0.])])
-        except:
+        if not issubclass(self.optimizer, torch.optim.Optimizer):
             raise ValueError(
                 f"{self.optimizer} is not a valid PyTorch optimizer."
             )
@@ -334,7 +332,7 @@ class ParametricDR(utils.repr._ReprMixin):
 
     Attributes:
         device: The device on which the model is allocated (depends on the
-            value specified for :attr:`use_cuda`).
+            value specified for ``use_cuda``).
     """
 
     def __init__(self,
@@ -420,11 +418,6 @@ class ParametricDR(utils.repr._ReprMixin):
         if use_cuda:
             self.model.cuda()
 
-        # set device to device of first model parameter
-        for p in self.model.parameters():
-            self.device = p.device
-            break
-
         self.trained = False
 
     @property
@@ -436,6 +429,14 @@ class ParametricDR(utils.repr._ReprMixin):
                 "Attempted to access a dataset, but none was registered."
             )
 
+    @property
+    def device(self) -> torch.device:
+        device = torch.device('cpu')
+        for p in self.model.parameters():
+            device = p.device
+            break
+        return device
+
     def __call__(self,
         X: TensorLike
     ) -> torch.Tensor: 
@@ -522,7 +523,7 @@ class ParametricDR(utils.repr._ReprMixin):
 
     def set_training_defaults(self,
         training_phase: Optional[TrainingPhase] = None,
-        n_epochs: Optional[int] = None,
+        epochs: Optional[int] = None,
         batch_size: Optional[int] = None,
         batches_per_epoch: Optional[int] = None,
         sampling: Optional[Literal['standard', 'negative_edge']] = None,
@@ -551,8 +552,8 @@ class ParametricDR(utils.repr._ReprMixin):
         """
         if training_phase is not None:
             self.training_defaults = copy.deepcopy(training_phase)
-        if n_epochs is not None:
-            self.training_defaults.n_epochs = n_epochs
+        if epochs is not None:
+            self.training_defaults.epochs = epochs
         if batch_size is not None:
             self.training_defaults.batch_size = batch_size
         if batches_per_epoch is not None:
@@ -580,7 +581,7 @@ class ParametricDR(utils.repr._ReprMixin):
     def add_training_phase(self,
         training_phase: Optional[TrainingPhase] = None,
         name: Optional[str] = None,
-        n_epochs: Optional[int] = None,
+        epochs: Optional[int] = None,
         batch_size: Optional[int] = None,
         batches_per_epoch: Optional[int] = None,
         sampling: Optional[Literal['standard', 'negative_edge']] = None,
@@ -617,8 +618,8 @@ class ParametricDR(utils.repr._ReprMixin):
 
         if name is not None:
             training_phase.name = name
-        if n_epochs is not None:
-            training_phase.n_epochs = n_epochs
+        if epochs is not None:
+            training_phase.epochs = epochs
         if batch_size is not None:
             training_phase.batch_size = batch_size
         if batches_per_epoch is not None:
@@ -694,23 +695,34 @@ class ParametricDR(utils.repr._ReprMixin):
                     utils.logging.log(f"Adding entry '{k}' to dataset.")
             self.dataset.data[k] = utils.convert.to_torch(data[k])
 
-    def _compute_global_relations(self) -> None:
+    def compute_global_relations(self, force: bool = False) -> None:
+        """Computes the global relations.
+        
+        The computed relation data are stored in the instance's 
+        ``global_relation_data`` attribute.
+
+        Args:
+            force: Whether or not to force a new computation, when relations
+                have been previously computed for the same instance.
+        """
 
         if not self._dataset_registered:
             raise exceptions.NoDatasetRegisteredError(
                 "Cannot compute global relations before registering dataset."
             )
         assert isinstance(self.dataset, Dataset)
+
+        if force or not self._global_relations_computed:
         
-        for k in self.global_relations:
-            if self.verbose:
-                utils.logging.log(
-                    f"Computing global relations '{k}'."
+            for k in self.global_relations:
+                if self.verbose:
+                    utils.logging.log(
+                        f"Computing global relations '{k}'."
+                    )
+                rel = self.global_relations[k]
+                self.global_relation_data[k] = rel.compute_relations(
+                    self.dataset.data[rel.data_key]
                 )
-            self.global_relation_data[k] = (
-                self.global_relations[k].compute_relations(
-                    self.dataset.data['data']
-                ))
                 
         self._global_relations_computed = True
 
@@ -801,12 +813,14 @@ class ParametricDR(utils.repr._ReprMixin):
         dataloader = self._prepare_loader(training_phase)
         optimizer = self._prepare_optimizer(training_phase)
 
+        device = self.device
+
         if self.verbose:
             utils.logging.log(
                 f"Beginning training phase '{training_phase.name}'."
             )
 
-        for epoch in range(training_phase.n_epochs):
+        for epoch in range(training_phase.epochs):
             
             batch: dict[str, torch.Tensor]
             for batch in dataloader:
@@ -818,7 +832,7 @@ class ParametricDR(utils.repr._ReprMixin):
                     self.global_relation_data,
                     self.batch_relations,
                     batch,
-                    self.device
+                    device
                 )
 
                 loss.backward()
@@ -841,8 +855,7 @@ class ParametricDR(utils.repr._ReprMixin):
         routine.
         """
         self._prepare_training()
-        if not self._global_relations_computed:
-            self._compute_global_relations()
+        self.compute_global_relations()
 
         self.model.train()
 
